@@ -583,9 +583,7 @@ public enum PGProperty {
 
   /**
    * The String argument to give to the constructor of the Socket Factory.
-   * @deprecated use {@code ..Factory(Properties)} constructor.
    */
-  @Deprecated
   SOCKET_FACTORY_ARG(
       "socketFactoryArg",
       null,
@@ -925,7 +923,7 @@ public enum PGProperty {
    * @return evaluated value for this connection parameter converted to int
    * @throws NumberFormatException if it cannot be converted to int.
    */
-  @SuppressWarnings("nullness:argument.type.incompatible")
+  @SuppressWarnings("nullness:argument")
   public int getIntNoCheck(Properties properties) {
     String value = getOrDefault(properties);
     //noinspection ConstantConditions
@@ -939,7 +937,7 @@ public enum PGProperty {
    * @return evaluated value for this connection parameter converted to int
    * @throws PSQLException if it cannot be converted to int.
    */
-  @SuppressWarnings("nullness:argument.type.incompatible")
+  @SuppressWarnings("nullness:argument")
   public int getInt(Properties properties) throws PSQLException {
     String value = getOrDefault(properties);
     try {
