@@ -173,59 +173,6 @@ class SimpleParameterList implements V3ParameterList {
     bind(index, NULL_OBJECT, oid, binaryTransfer);
   }
 
-  /**
-   * <p>Escapes a given text value as a literal, wraps it in single quotes, casts it to the
-   * to the given data type, and finally wraps the whole thing in parentheses.</p>
-   *
-   * <p>For example, "123" and "int4" becomes "('123'::int)"</p>
-   *
-   * <p>The additional parentheses is added to ensure that the surrounding text of where the
-   * parameter value is entered does modify the interpretation of the value.</p>
-   *
-   * <p>For example if our input SQL is: <code>SELECT ?b</code></p>
-   *
-   * <p>Using a parameter value of '{}' and type of json we'd get:</p>
-   *
-   * <pre>
-   * test=# SELECT ('{}'::json)b;
-   *  b
-   * ----
-   *  {}
-   * </pre>
-   *
-   * <p>But without the parentheses the result changes:</p>
-   *
-   * <pre>
-   * test=# SELECT '{}'::jsonb;
-   * jsonb
-   * -------
-   * {}
-   * </pre>
-   **/
-  private static String quoteAndCast(String text, /* @Nullable */ String type, boolean standardConformingStrings) {
-
-    StringBuilder sb = new StringBuilder((text.length() + 10) / 10 * 11); // Add 10% for escaping.
-    sb.append("('");
-    try {
-      Utils.escapeLiteral(sb, text, standardConformingStrings);
-    } catch (SQLException e) {
-      // This should only happen if we have an embedded null
-      // and there's not much we can do if we do hit one.
-      //
-      // To force a server side failure, we deliberately include
-      // a zero byte character in the literal to force the server
-      // to reject the command.
-      sb.append('\u0000');
-    }
-    sb.append("'");
-    if (type != null) {
-      sb.append("::");
-      sb.append(type);
-    }
-    sb.append(")");
-    return sb.toString();
-  }
-
   @Override
   public String toString(/* @Positive */ int index, boolean standardConformingStrings) {
     --index;
@@ -233,103 +180,101 @@ class SimpleParameterList implements V3ParameterList {
     if (paramValue == null) {
       return "?";
     } else if (paramValue == NULL_OBJECT) {
-      return "(NULL)";
-    }
-    String textValue;
-    String type;
-    if ((flags[index] & BINARY) == BINARY) {
+      return "NULL";
+    } else if ((flags[index] & BINARY) == BINARY) {
       // handle some of the numeric types
+
       switch (paramTypes[index]) {
         case Oid.INT2:
           short s = ByteConverter.int2((byte[]) paramValue, 0);
-          textValue = Short.toString(s);
-          type = "int2";
-          break;
+          return Short.toString(s);
 
         case Oid.INT4:
           int i = ByteConverter.int4((byte[]) paramValue, 0);
-          textValue = Integer.toString(i);
-          type = "int4";
-          break;
+          return Integer.toString(i);
 
         case Oid.INT8:
           long l = ByteConverter.int8((byte[]) paramValue, 0);
-          textValue = Long.toString(l);
-          type = "int8";
-          break;
+          return Long.toString(l);
 
         case Oid.FLOAT4:
           float f = ByteConverter.float4((byte[]) paramValue, 0);
           if (Float.isNaN(f)) {
-            return "('NaN'::real)";
+            return "'NaN'::real";
           }
-          textValue = Float.toString(f);
-          type = "real";
-          break;
+          return Float.toString(f);
 
         case Oid.FLOAT8:
           double d = ByteConverter.float8((byte[]) paramValue, 0);
           if (Double.isNaN(d)) {
-            return "('NaN'::double precision)";
+            return "'NaN'::double precision";
           }
-          textValue = Double.toString(d);
-          type = "double precision";
-          break;
+          return Double.toString(d);
 
         case Oid.NUMERIC:
           Number n = ByteConverter.numeric((byte[]) paramValue);
           if (n instanceof Double) {
             assert ((Double) n).isNaN();
-            return "('NaN'::numeric)";
+            return "'NaN'::numeric";
           }
-          textValue = n.toString();
-          type = "numeric";
-          break;
+          return n.toString();
 
         case Oid.UUID:
-          textValue =
+          String uuid =
               new UUIDArrayAssistant().buildElement((byte[]) paramValue, 0, 16).toString();
-          type = "uuid";
-          break;
+          return "'" + uuid + "'::uuid";
+
         case Oid.POINT:
           PGpoint pgPoint = new PGpoint();
           pgPoint.setByteValue((byte[]) paramValue, 0);
-          textValue = pgPoint.toString();
-          type = "point";
-          break;
+          return "'" + pgPoint.toString() + "'::point";
 
         case Oid.BOX:
           PGbox pgBox = new PGbox();
           pgBox.setByteValue((byte[]) paramValue, 0);
-          textValue = pgBox.toString();
-          type = "box";
-          break;
-
-        default:
-          return "?";
+          return "'" + pgBox.toString() + "'::box";
       }
+      return "?";
     } else {
-      textValue = paramValue.toString();
+      String param = paramValue.toString();
+
+      // add room for quotes + potential escaping.
+      StringBuilder p = new StringBuilder(3 + (param.length() + 10) / 10 * 11);
+
+      // No E'..' here since escapeLiteral escapes all things and it does not use \123 kind of
+      // escape codes
+      p.append('\'');
+      try {
+        p = Utils.escapeLiteral(p, param, standardConformingStrings);
+      } catch (SQLException sqle) {
+        // This should only happen if we have an embedded null
+        // and there's not much we can do if we do hit one.
+        //
+        // The goal of toString isn't to be sent to the server,
+        // so we aren't 100% accurate (see StreamWrapper), put
+        // the unescaped version of the data.
+        //
+        p.append(param);
+      }
+      p.append('\'');
       int paramType = paramTypes[index];
       if (paramType == Oid.TIMESTAMP) {
-        type = "timestamp";
+        p.append("::timestamp");
       } else if (paramType == Oid.TIMESTAMPTZ) {
-        type = "timestamp with time zone";
+        p.append("::timestamp with time zone");
       } else if (paramType == Oid.TIME) {
-        type = "time";
+        p.append("::time");
       } else if (paramType == Oid.TIMETZ) {
-        type = "time with time zone";
+        p.append("::time with time zone");
       } else if (paramType == Oid.DATE) {
-        type = "date";
+        p.append("::date");
       } else if (paramType == Oid.INTERVAL) {
-        type = "interval";
+        p.append("::interval");
       } else if (paramType == Oid.NUMERIC) {
-        type = "numeric";
-      } else {
-        type = null;
+        p.append("::numeric");
       }
+      return p.toString();
     }
-    return quoteAndCast(textValue, type, standardConformingStrings);
   }
 
   @Override
@@ -396,8 +341,8 @@ class SimpleParameterList implements V3ParameterList {
   }
 
   void setResolvedType(/* @Positive */ int index, int oid) {
-    // only allow overwriting an unknown value
-    if (paramTypes[index - 1] == Oid.UNSPECIFIED) {
+    // only allow overwriting an unknown value or VOID value
+    if (paramTypes[index - 1] == Oid.UNSPECIFIED || paramTypes[index - 1] == Oid.VOID) {
       paramTypes[index - 1] = oid;
     } else if (paramTypes[index - 1] != oid) {
       throw new IllegalArgumentException("Can't change resolved type for param: " + index + " from "
@@ -468,7 +413,9 @@ class SimpleParameterList implements V3ParameterList {
 
     // Binary-format bytea?
     if (paramValue instanceof StreamWrapper) {
-      streamBytea(pgStream, (StreamWrapper) paramValue);
+      try (StreamWrapper streamWrapper = (StreamWrapper) paramValue;) {
+        streamBytea(pgStream, streamWrapper);
+      }
       return;
     }
 
