@@ -663,6 +663,7 @@ public abstract class BaseDataSource implements CommonDataSource, Referenceable
    * @return SSL mode
    * @see PGProperty#SSL_FACTORY_ARG
    */
+  @SuppressWarnings("deprecation")
   public /* @Nullable */ String getSslFactoryArg() {
     return PGProperty.SSL_FACTORY_ARG.getOrDefault(properties);
   }
@@ -671,6 +672,7 @@ public abstract class BaseDataSource implements CommonDataSource, Referenceable
    * @param arg argument forwarded to SSL factory
    * @see PGProperty#SSL_FACTORY_ARG
    */
+  @SuppressWarnings("deprecation")
   public void setSslFactoryArg(/* @Nullable */ String arg) {
     PGProperty.SSL_FACTORY_ARG.set(properties, arg);
   }
@@ -1219,6 +1221,7 @@ public abstract class BaseDataSource implements CommonDataSource, Referenceable
    * @return socket factory argument
    * @see PGProperty#SOCKET_FACTORY_ARG
    */
+  @SuppressWarnings("deprecation")
   public /* @Nullable */ String getSocketFactoryArg() {
     return PGProperty.SOCKET_FACTORY_ARG.getOrDefault(properties);
   }
@@ -1227,6 +1230,7 @@ public abstract class BaseDataSource implements CommonDataSource, Referenceable
    * @param socketFactoryArg socket factory argument
    * @see PGProperty#SOCKET_FACTORY_ARG
    */
+  @SuppressWarnings("deprecation")
   public void setSocketFactoryArg(/* @Nullable */ String socketFactoryArg) {
     PGProperty.SOCKET_FACTORY_ARG.set(properties, socketFactoryArg);
   }
@@ -1333,8 +1337,16 @@ public abstract class BaseDataSource implements CommonDataSource, Referenceable
         url.append(",");
       }
       url.append(serverNames[i]);
-      if (portNumbers != null && portNumbers.length >= i && portNumbers[i] != 0) {
-        url.append(":").append(portNumbers[i]);
+      if (portNumbers != null) {
+        if (serverNames.length != portNumbers.length) {
+          throw new IllegalArgumentException(
+              String.format("Invalid argument: number of port %s entries must equal number of serverNames %s",
+                  Arrays.toString(portNumbers), Arrays.toString(serverNames)));
+        }
+        if (portNumbers.length >= i && portNumbers[i] != 0) {
+          url.append(":").append(portNumbers[i]);
+        }
+
       }
     }
     url.append("/");
