@@ -53,7 +53,7 @@ public class PGcircle extends PGobject implements Serializable, Cloneable {
    * @param s definition of the circle in PostgreSQL's syntax.
    * @throws SQLException on conversion failure
    */
-  @SuppressWarnings("method.invocation.invalid")
+  @SuppressWarnings("method.invocation")
   public PGcircle(String s) throws SQLException {
     this();
     setValue(s);
