@@ -51,7 +51,7 @@ public class PGpath extends PGobject implements Serializable, Cloneable {
    * @param s definition of the path in PostgreSQL's syntax.
    * @throws SQLException on conversion failure
    */
-  @SuppressWarnings("method.invocation.invalid")
+  @SuppressWarnings("method.invocation")
   public PGpath(String s) throws SQLException {
     this();
     setValue(s);
