@@ -57,7 +57,7 @@ public class PGpoint extends PGobject implements PGBinaryObject, Serializable, C
    * @param value Definition of this point in PostgreSQL's syntax
    * @throws SQLException if something goes wrong
    */
-  @SuppressWarnings("method.invocation.invalid")
+  @SuppressWarnings("method.invocation")
   public PGpoint(String value) throws SQLException {
     this();
     setValue(value);
@@ -207,7 +207,10 @@ public class PGpoint extends PGobject implements PGBinaryObject, Serializable, C
    *
    * @param p Point to move to
    * @see java.awt.Point
+   *
+   * @deprecated Will be removed for avoiding a dependency to the {@code java.desktop} module.
    */
+  @Deprecated
   public void setLocation(Point p) {
     setLocation(p.x, p.y);
   }
