@@ -7,7 +7,6 @@ package org.postgresql.jdbc;
 
 import static org.postgresql.util.internal.Nullness.castNonNull;
 
-import org.postgresql.Driver;
 import org.postgresql.PGNotification;
 import org.postgresql.PGProperty;
 import org.postgresql.copy.CopyManager;
@@ -33,6 +32,7 @@ import org.postgresql.replication.PGReplicationConnection;
 import org.postgresql.replication.PGReplicationConnectionImpl;
 import org.postgresql.util.GT;
 import org.postgresql.util.HostSpec;
+import org.postgresql.util.LazyCleaner;
 import org.postgresql.util.LruCache;
 import org.postgresql.util.PGBinaryObject;
 import org.postgresql.util.PGobject;
@@ -84,6 +84,7 @@ import java.util.StringTokenizer;
 import java.util.Timer;
 import java.util.TimerTask;
 import java.util.concurrent.Executor;
+import java.util.concurrent.locks.Condition;
 import java.util.logging.Level;
 import java.util.logging.Logger;
 
@@ -117,9 +118,12 @@ public class PgConnection implements BaseConnection {
   private enum ReadOnlyBehavior {
     ignore,
     transaction,
-    always;
+    always
   }
 
+  private final ResourceLock lock = new ResourceLock();
+  private final Condition lockCondition = lock.newCondition();
+
   //
   // Data initialized on construction:
   //
@@ -132,6 +136,15 @@ public class PgConnection implements BaseConnection {
 
   private /* @Nullable */ Throwable openStackTrace;
 
+  /**
+   * This field keeps finalize action alive, so its .finalize() method is called only
+   * when the connection itself becomes unreachable.
+   * Moving .finalize() to a different object allows JVM to release all the other objects
+   * referenced in PgConnection early.
+   */
+  private final PgConnectionCleaningAction finalizeAction;
+  private final Object leakHandle = new Object();
+
   /* Actual network handler */
   private final QueryExecutor queryExecutor;
 
@@ -182,10 +195,6 @@ public class PgConnection implements BaseConnection {
   // Current warnings; there might be more on queryExecutor too.
   private /* @Nullable */ SQLWarning firstWarning;
 
-  // Timer for scheduling TimerTasks for this connection.
-  // Only instantiated if a task is actually scheduled.
-  private volatile /* @Nullable */ Timer cancelTimer;
-
   private /* @Nullable */ PreparedStatement checkConnectionQuery;
   /**
    * Replication protocol in current version postgresql(10devel) supports a limited number of
@@ -197,6 +206,7 @@ public class PgConnection implements BaseConnection {
 
   private final /* @Nullable */ String xmlFactoryFactoryClass;
   private /* @Nullable */ PGXmlFactoryFactory xmlFactoryFactory;
+  private final LazyCleaner.Cleanable<IOException> cleanable;
 
   final CachedQuery borrowQuery(String sql) throws SQLException {
     return queryExecutor.borrowQuery(sql);
@@ -231,7 +241,7 @@ public class PgConnection implements BaseConnection {
   //
   // Ctor.
   //
-  @SuppressWarnings({"method.invocation.invalid", "argument.type.incompatible"})
+  @SuppressWarnings({"method.invocation"})
   public PgConnection(HostSpec[] hostSpecs,
                       Properties info,
                       String url) throws SQLException {
@@ -277,9 +287,9 @@ public class PgConnection implements BaseConnection {
     }
 
     // split for receive and send for better control
-    Set<Integer> useBinarySendForOids = new HashSet<Integer>(binaryOids);
+    Set<Integer> useBinarySendForOids = new HashSet<>(binaryOids);
 
-    Set<Integer> useBinaryReceiveForOids = new HashSet<Integer>(binaryOids);
+    Set<Integer> useBinaryReceiveForOids = new HashSet<>(binaryOids);
 
     /*
      * Does not pass unit tests because unit tests expect setDate to have millisecond accuracy
@@ -328,12 +338,15 @@ public class PgConnection implements BaseConnection {
     int unknownLength = PGProperty.UNKNOWN_LENGTH.getInt(info);
 
     // Initialize object handling
-    typeCache = createTypeInfo(this, unknownLength);
+    @SuppressWarnings("argument")
+    TypeInfo typeCache = createTypeInfo(this, unknownLength);
+    this.typeCache = typeCache;
     initObjectTypes(info);
 
     if (PGProperty.LOG_UNCLOSED_CONNECTIONS.getBoolean(info)) {
       openStackTrace = new Throwable("Connection was created at this point:");
     }
+    finalizeAction = new PgConnectionCleaningAction(lock, openStackTrace, queryExecutor.getCloseAction());
     this.logServerErrorDetail = PGProperty.LOG_SERVER_ERROR_DETAIL.getBoolean(info);
     this.disableColumnSanitiser = PGProperty.DISABLE_COLUMN_SANITISER.getBoolean(info);
 
@@ -351,17 +364,21 @@ public class PgConnection implements BaseConnection {
       this.clientInfo.put("ApplicationName", appName);
     }
 
-    fieldMetadataCache = new LruCache<FieldMetadata.Key, FieldMetadata>(
-            Math.max(0, PGProperty.DATABASE_METADATA_CACHE_FIELDS.getInt(info)),
-            Math.max(0, PGProperty.DATABASE_METADATA_CACHE_FIELDS_MIB.getInt(info) * 1024L * 1024L),
+    fieldMetadataCache = new LruCache<>(
+        Math.max(0, PGProperty.DATABASE_METADATA_CACHE_FIELDS.getInt(info)),
+        Math.max(0, PGProperty.DATABASE_METADATA_CACHE_FIELDS_MIB.getInt(info) * 1024L * 1024L),
         false);
 
     replicationConnection = PGProperty.REPLICATION.getOrDefault(info) != null;
 
     xmlFactoryFactoryClass = PGProperty.XML_FACTORY_FACTORY.getOrDefault(info);
+    cleanable = LazyCleaner.getInstance().register(leakHandle, finalizeAction);
   }
 
-  private static ReadOnlyBehavior getReadOnlyBehavior(String property) {
+  private static ReadOnlyBehavior getReadOnlyBehavior(/* @Nullable */ String property) {
+    if (property == null) {
+      return ReadOnlyBehavior.transaction;
+    }
     try {
       return ReadOnlyBehavior.valueOf(property);
     } catch (IllegalArgumentException e) {
@@ -374,7 +391,7 @@ public class PgConnection implements BaseConnection {
   }
 
   private static Set<Integer> getSupportedBinaryOids() {
-    return new HashSet<Integer>(Arrays.asList(
+    return new HashSet<>(Arrays.asList(
         Oid.BYTEA,
         Oid.INT2,
         Oid.INT4,
@@ -412,7 +429,7 @@ public class PgConnection implements BaseConnection {
     // check if binary transfer should be enabled for built-in types
     boolean binaryTransfer = PGProperty.BINARY_TRANSFER.getBoolean(info);
     // get formats that currently have binary protocol support
-    Set<Integer> binaryOids = new HashSet<Integer>(32);
+    Set<Integer> binaryOids = new HashSet<>(32);
     if (binaryTransfer) {
       binaryOids.addAll(SUPPORTED_BINARY_OIDS);
     }
@@ -478,7 +495,22 @@ public class PgConnection implements BaseConnection {
   /**
    * The current type mappings.
    */
-  protected Map<String, Class<?>> typemap = new HashMap<String, Class<?>>();
+  protected Map<String, Class<?>> typemap = new HashMap<>();
+
+  /**
+   * Obtain the connection lock and return it. Callers must use try-with-resources to ensure that
+   * unlock() is performed on the lock.
+   */
+  final ResourceLock obtainLock() {
+    return lock.obtain();
+  }
+
+  /**
+   * Return the lock condition for this connection.
+   */
+  final Condition lockCondition() {
+    return lockCondition;
+  }
 
   @Override
   public Statement createStatement() throws SQLException {
@@ -556,39 +588,37 @@ public class PgConnection implements BaseConnection {
 
   @Override
   public void execSQLUpdate(String s) throws SQLException {
-    BaseStatement stmt = (BaseStatement) createStatement();
-    if (stmt.executeWithFlags(s, QueryExecutor.QUERY_NO_METADATA | QueryExecutor.QUERY_NO_RESULTS
-        | QueryExecutor.QUERY_SUPPRESS_BEGIN)) {
-      throw new PSQLException(GT.tr("A result was returned when none was expected."),
-          PSQLState.TOO_MANY_RESULTS);
-    }
+    try (BaseStatement stmt = (BaseStatement) createStatement()) {
+      if (stmt.executeWithFlags(s, QueryExecutor.QUERY_NO_METADATA | QueryExecutor.QUERY_NO_RESULTS
+          | QueryExecutor.QUERY_SUPPRESS_BEGIN)) {
+        throw new PSQLException(GT.tr("A result was returned when none was expected."),
+            PSQLState.TOO_MANY_RESULTS);
+      }
 
-    // Transfer warnings to the connection, since the user never
-    // has a chance to see the statement itself.
-    SQLWarning warnings = stmt.getWarnings();
-    if (warnings != null) {
-      addWarning(warnings);
+      // Transfer warnings to the connection, since the user never
+      // has a chance to see the statement itself.
+      SQLWarning warnings = stmt.getWarnings();
+      if (warnings != null) {
+        addWarning(warnings);
+      }
     }
-
-    stmt.close();
   }
 
   void execSQLUpdate(CachedQuery query) throws SQLException {
-    BaseStatement stmt = (BaseStatement) createStatement();
-    if (stmt.executeWithFlags(query, QueryExecutor.QUERY_NO_METADATA | QueryExecutor.QUERY_NO_RESULTS
-        | QueryExecutor.QUERY_SUPPRESS_BEGIN)) {
-      throw new PSQLException(GT.tr("A result was returned when none was expected."),
-          PSQLState.TOO_MANY_RESULTS);
-    }
+    try (BaseStatement stmt = (BaseStatement) createStatement()) {
+      if (stmt.executeWithFlags(query, QueryExecutor.QUERY_NO_METADATA | QueryExecutor.QUERY_NO_RESULTS
+          | QueryExecutor.QUERY_SUPPRESS_BEGIN)) {
+        throw new PSQLException(GT.tr("A result was returned when none was expected."),
+            PSQLState.TOO_MANY_RESULTS);
+      }
 
-    // Transfer warnings to the connection, since the user never
-    // has a chance to see the statement itself.
-    SQLWarning warnings = stmt.getWarnings();
-    if (warnings != null) {
-      addWarning(warnings);
+      // Transfer warnings to the connection, since the user never
+      // has a chance to see the statement itself.
+      SQLWarning warnings = stmt.getWarnings();
+      if (warnings != null) {
+        addWarning(warnings);
+      }
     }
-
-    stmt.close();
   }
 
   /**
@@ -640,6 +670,7 @@ public class PgConnection implements BaseConnection {
     return queryExecutor.getUser();
   }
 
+  @SuppressWarnings("deprecation") // support for deprecated Fastpath API
   public Fastpath getFastpathAPI() throws SQLException {
     checkClosed();
     if (fastpath == null) {
@@ -649,6 +680,7 @@ public class PgConnection implements BaseConnection {
   }
 
   // This holds a reference to the Fastpath API if already open
+  @SuppressWarnings("deprecation") // support for deprecated Fastpath API
   private /* @Nullable */ Fastpath fastpath;
 
   public LargeObjectManager getLargeObjectAPI() throws SQLException {
@@ -704,7 +736,7 @@ public class PgConnection implements BaseConnection {
       // point, etc).
 
       if (klass != null) {
-        obj = klass.newInstance();
+        obj = klass.getDeclaredConstructor().newInstance();
         obj.setType(type);
         if (byteValue != null && obj instanceof PGBinaryObject) {
           PGBinaryObject binObj = (PGBinaryObject) obj;
@@ -805,7 +837,6 @@ public class PgConnection implements BaseConnection {
    * <B>Note:</B> even though {@code Statement} is automatically closed when it is garbage
    * collected, it is better to close it explicitly to lower resource consumption.
    * The spec says that calling close on a closed connection is a no-op.
-   *
    * {@inheritDoc}
    */
   @Override
@@ -815,12 +846,14 @@ public class PgConnection implements BaseConnection {
       // When that happens the connection is still registered in the finalizer queue, so it gets finalized
       return;
     }
-    if (queryExecutor.isClosed()) {
-      return;
-    }
-    releaseTimer();
-    queryExecutor.close();
     openStackTrace = null;
+    try {
+      cleanable.clean();
+    } catch (IOException e) {
+      throw new PSQLException(
+          GT.tr("Unable to close connection properly"),
+          PSQLState.UNKNOWN_STATE, e);
+    }
   }
 
   @Override
@@ -832,24 +865,28 @@ public class PgConnection implements BaseConnection {
   }
 
   @Override
-  public synchronized /* @Nullable */ SQLWarning getWarnings() throws SQLException {
-    checkClosed();
-    SQLWarning newWarnings = queryExecutor.getWarnings(); // NB: also clears them.
-    if (firstWarning == null) {
-      firstWarning = newWarnings;
-    } else if (newWarnings != null) {
-      firstWarning.setNextWarning(newWarnings); // Chain them on.
-    }
+  public /* @Nullable */ SQLWarning getWarnings() throws SQLException {
+    try (ResourceLock ignore = lock.obtain()) {
+      checkClosed();
+      SQLWarning newWarnings = queryExecutor.getWarnings(); // NB: also clears them.
+      if (firstWarning == null) {
+        firstWarning = newWarnings;
+      } else if (newWarnings != null) {
+        firstWarning.setNextWarning(newWarnings); // Chain them on.
+      }
 
-    return firstWarning;
+      return firstWarning;
+    }
   }
 
   @Override
-  public synchronized void clearWarnings() throws SQLException {
-    checkClosed();
-    //noinspection ThrowableNotThrown
-    queryExecutor.getWarnings(); // Clear and discard.
-    firstWarning = null;
+  public void clearWarnings() throws SQLException {
+    try (ResourceLock ignore = lock.obtain()) {
+      checkClosed();
+      //noinspection ThrowableNotThrown
+      queryExecutor.getWarnings(); // Clear and discard.
+      firstWarning = null;
+    }
   }
 
   @Override
@@ -1061,25 +1098,6 @@ public class PgConnection implements BaseConnection {
     return hideUnprivilegedObjects;
   }
 
-  /**
-   * <p>Overrides finalize(). If called, it closes the connection.</p>
-   *
-   * <p>This was done at the request of <a href="mailto:rachel@enlarion.demon.co.uk">Rachel
-   * Greenham</a> who hit a problem where multiple clients didn't close the connection, and once a
-   * fortnight enough clients were open to kill the postgres server.</p>
-   */
-  protected void finalize() throws Throwable {
-    try {
-      if (openStackTrace != null) {
-        LOGGER.log(Level.WARNING, GT.tr("Finalizing a Connection that was never closed:"), openStackTrace);
-      }
-
-      close();
-    } finally {
-      super.finalize();
-    }
-  }
-
   /**
    * Get server version number.
    *
@@ -1291,18 +1309,8 @@ public class PgConnection implements BaseConnection {
     queryExecutor.abort();
   }
 
-  private synchronized Timer getTimer() {
-    if (cancelTimer == null) {
-      cancelTimer = Driver.getSharedTimer().getTimer();
-    }
-    return cancelTimer;
-  }
-
-  private synchronized void releaseTimer() {
-    if (cancelTimer != null) {
-      cancelTimer = null;
-      Driver.getSharedTimer().releaseTimer();
-    }
+  private Timer getTimer() {
+    return finalizeAction.getTimer();
   }
 
   @Override
@@ -1313,10 +1321,7 @@ public class PgConnection implements BaseConnection {
 
   @Override
   public void purgeTimerTasks() {
-    Timer timer = cancelTimer;
-    if (timer != null) {
-      timer.purge();
-    }
+    finalizeAction.purgeTimerTasks();
   }
 
   @Override
@@ -1504,7 +1509,7 @@ public class PgConnection implements BaseConnection {
           statement.close();
         } else {
           PreparedStatement checkConnectionQuery;
-          synchronized (this) {
+          try (ResourceLock ignore = lock.obtain()) {
             checkConnectionQuery = this.checkConnectionQuery;
             if (checkConnectionQuery == null) {
               checkConnectionQuery = prepareStatement("");
@@ -1534,7 +1539,7 @@ public class PgConnection implements BaseConnection {
     try {
       checkClosed();
     } catch (final SQLException cause) {
-      Map<String, ClientInfoStatus> failures = new HashMap<String, ClientInfoStatus>();
+      Map<String, ClientInfoStatus> failures = new HashMap<>();
       failures.put(name, ClientInfoStatus.REASON_UNKNOWN);
       throw new SQLClientInfoException(GT.tr("This connection has been closed."), failures, cause);
     }
@@ -1554,7 +1559,7 @@ public class PgConnection implements BaseConnection {
         sql.append("'");
         execSQLUpdate(sql.toString());
       } catch (SQLException sqle) {
-        Map<String, ClientInfoStatus> failures = new HashMap<String, ClientInfoStatus>();
+        Map<String, ClientInfoStatus> failures = new HashMap<>();
         failures.put(name, ClientInfoStatus.REASON_UNKNOWN);
         throw new SQLClientInfoException(
             GT.tr("Failed to set ClientInfo property: {0}", "ApplicationName"), sqle.getSQLState(),
@@ -1576,14 +1581,14 @@ public class PgConnection implements BaseConnection {
     try {
       checkClosed();
     } catch (final SQLException cause) {
-      Map<String, ClientInfoStatus> failures = new HashMap<String, ClientInfoStatus>();
+      Map<String, ClientInfoStatus> failures = new HashMap<>();
       for (Map.Entry<Object, Object> e : properties.entrySet()) {
         failures.put((String) e.getKey(), ClientInfoStatus.REASON_UNKNOWN);
       }
       throw new SQLClientInfoException(GT.tr("This connection has been closed."), failures, cause);
     }
 
-    Map<String, ClientInfoStatus> failures = new HashMap<String, ClientInfoStatus>();
+    Map<String, ClientInfoStatus> failures = new HashMap<>();
     for (String name : new String[]{"ApplicationName"}) {
       try {
         setClientInfo(name, properties.getProperty(name, null));
@@ -1638,26 +1643,19 @@ public class PgConnection implements BaseConnection {
 
   public /* @Nullable */ String getSchema() throws SQLException {
     checkClosed();
-    Statement stmt = createStatement();
-    try {
-      ResultSet rs = stmt.executeQuery("select current_schema()");
-      try {
+    try (Statement stmt = createStatement()) {
+      try (ResultSet rs = stmt.executeQuery("select current_schema()")) {
         if (!rs.next()) {
           return null; // Is it ever possible?
         }
         return rs.getString(1);
-      } finally {
-        rs.close();
       }
-    } finally {
-      stmt.close();
     }
   }
 
   public void setSchema(/* @Nullable */ String schema) throws SQLException {
     checkClosed();
-    Statement stmt = createStatement();
-    try {
+    try (Statement stmt = createStatement()) {
       if (schema == null) {
         stmt.executeUpdate("SET SESSION search_path TO DEFAULT");
       } else {
@@ -1668,8 +1666,6 @@ public class PgConnection implements BaseConnection {
         stmt.executeUpdate(sb.toString());
         LOGGER.log(Level.FINE, "  setSchema = {0}", schema);
       }
-    } finally {
-      stmt.close();
     }
   }
 
@@ -1742,8 +1738,6 @@ public class PgConnection implements BaseConnection {
 
     switch (holdability) {
       case ResultSet.CLOSE_CURSORS_AT_COMMIT:
-        rsHoldability = holdability;
-        break;
       case ResultSet.HOLD_CURSORS_OVER_COMMIT:
         rsHoldability = holdability;
         break;
@@ -1927,7 +1921,9 @@ public class PgConnection implements BaseConnection {
             PSQLState.INVALID_PARAMETER_VALUE);
       }
       try {
-        xmlFactoryFactory = (PGXmlFactoryFactory) clazz.newInstance();
+        xmlFactoryFactory = clazz.asSubclass(PGXmlFactoryFactory.class)
+            .getDeclaredConstructor()
+            .newInstance();
       } catch (Exception ex) {
         throw new PSQLException(
             GT.tr("Could not instantiate xmlFactoryFactory: {0}", xmlFactoryFactoryClass),
