@@ -88,7 +88,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     this(connection, connection.borrowQuery(sql), rsType, rsConcurrency, rsHoldability);
   }
 
-  @SuppressWarnings("method.invocation.invalid")
+  @SuppressWarnings("method.invocation")
   PgPreparedStatement(PgConnection connection, CachedQuery query, int rsType,
       int rsConcurrency, int rsHoldability) throws SQLException {
     super(connection, rsType, rsConcurrency, rsHoldability);
@@ -130,7 +130,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
    */
   @Override
   public ResultSet executeQuery() throws SQLException {
-    synchronized (this) {
+    try (ResourceLock ignore = lock.obtain()) {
       if (!executeWithFlags(0)) {
         throw new PSQLException(GT.tr("No results were returned by the query."), PSQLState.NO_DATA);
       }
@@ -148,7 +148,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
 
   @Override
   public int executeUpdate() throws SQLException {
-    synchronized (this) {
+    try (ResourceLock ignore = lock.obtain()) {
       executeWithFlags(QueryExecutor.QUERY_NO_RESULTS);
       checkNoResultUpdate();
       return getUpdateCount();
@@ -157,7 +157,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
 
   @Override
   public long executeLargeUpdate() throws SQLException {
-    synchronized (this) {
+    try (ResourceLock ignore = lock.obtain()) {
       executeWithFlags(QueryExecutor.QUERY_NO_RESULTS);
       checkNoResultUpdate();
       return getLargeUpdateCount();
@@ -173,14 +173,14 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
 
   @Override
   public boolean execute() throws SQLException {
-    synchronized (this) {
+    try (ResourceLock ignore = lock.obtain()) {
       return executeWithFlags(0);
     }
   }
 
   public boolean executeWithFlags(int flags) throws SQLException {
     try {
-      synchronized (this) {
+      try (ResourceLock ignore = lock.obtain()) {
         checkClosed();
 
         if (connection.getPreferQueryMode() == PreferQueryMode.SIMPLE) {
@@ -472,6 +472,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     setCharacterStreamPost71(parameterIndex, x, length, "ASCII");
   }
 
+  @SuppressWarnings("deprecation")
   public void setUnicodeStream(/* @Positive */ int parameterIndex, /* @Nullable */ InputStream x,
       /* @NonNegative */ int length) throws SQLException {
     checkClosed();
@@ -689,7 +690,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
         if (in instanceof Blob) {
           setBlob(parameterIndex, (Blob) in);
         } else if (in instanceof InputStream) {
-          long oid = createBlob(parameterIndex, (InputStream) in, -1);
+          long oid = createBlob(parameterIndex, (InputStream) in, Long.MAX_VALUE);
           setLong(parameterIndex, oid);
         } else {
           throw new PSQLException(
@@ -1197,31 +1198,23 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
     LargeObjectManager lom = connection.getLargeObjectAPI();
     long oid = lom.createLO();
     LargeObject lob = lom.open(oid);
-    OutputStream outputStream = lob.getOutputStream();
-    byte[] buf = new byte[4096];
-    try {
-      long remaining;
-      if (length > 0) {
-        remaining = length;
-      } else {
-        remaining = Long.MAX_VALUE;
-      }
-      int numRead = inputStream.read(buf, 0,
-          (length > 0 && remaining < buf.length ? (int) remaining : buf.length));
-      while (numRead != -1 && remaining > 0) {
-        remaining -= numRead;
+    try (OutputStream outputStream = lob.getOutputStream();) {
+      // The actual buffer size does not matter much, see benchmarks
+      // https://github.com/pgjdbc/pgjdbc/pull/3044#issuecomment-1838057929
+      // BlobOutputStream would gradually increase the buffer, so it will level the number of
+      // database calls.
+      // At the same time, inputStream.read might produce less rows than requested, so we can not
+      // use a plain lob.write(buf, 0, numRead) as it might not align with 2K boundaries.
+      byte[] buf = new byte[(int) Math.min(length, 8192)];
+      int numRead;
+      while (length > 0 && (
+          numRead = inputStream.read(buf, 0, (int) Math.min(buf.length, length))) >= 0) {
+        length -= numRead;
         outputStream.write(buf, 0, numRead);
-        numRead = inputStream.read(buf, 0,
-            (length > 0 && remaining < buf.length ? (int) remaining : buf.length));
       }
     } catch (IOException se) {
       throw new PSQLException(GT.tr("Unexpected error writing large object to database."),
           PSQLState.UNEXPECTED_ERROR, se);
-    } finally {
-      try {
-        outputStream.close();
-      } catch (Exception e) {
-      }
     }
     return oid;
   }
@@ -1628,7 +1621,7 @@ class PgPreparedStatement extends PgStatement implements PreparedStatement {
       return;
     }
 
-    long oid = createBlob(parameterIndex, inputStream, -1);
+    long oid = createBlob(parameterIndex, inputStream, Long.MAX_VALUE);
     setLong(parameterIndex, oid);
   }
 
