@@ -13,6 +13,7 @@ import org.postgresql.core.TransactionState;
 import org.postgresql.core.Version;
 import org.postgresql.jdbc.GSSEncMode;
 import org.postgresql.jdbc.PgConnection;
+import org.postgresql.jdbc.ResourceLock;
 import org.postgresql.util.PSQLException;
 
 // import org.checkerframework.checker.nullness.qual.Nullable;
@@ -51,11 +52,17 @@ public class TestUtil {
   public static final String SERVER_HOST_PORT_PROP = "_test_hostport";
   public static final String DATABASE_PROP = "_test_database";
 
+  private static final ResourceLock lock = new ResourceLock();
+
   /*
    * Returns the Test database JDBC URL
    */
   public static String getURL() {
-    return getURL(getServer(), + getPort());
+    return getURL(getServer(), getPort());
+  }
+
+  public static String getURL(String database) {
+    return getURL(getServer() + ":" + getPort(), database);
   }
 
   public static String getURL(String server, int port) {
@@ -244,9 +251,11 @@ public class TestUtil {
 
   private static Properties sslTestProperties = null;
 
-  private static synchronized void initSslTestProperties() {
-    if (sslTestProperties == null) {
-      sslTestProperties = TestUtil.loadPropertyFiles("ssltest.properties");
+  private static void initSslTestProperties() {
+    try (ResourceLock ignore = lock.obtain()) {
+      if (sslTestProperties == null) {
+        sslTestProperties = TestUtil.loadPropertyFiles("ssltest.properties");
+      }
     }
   }
 
@@ -265,7 +274,7 @@ public class TestUtil {
   }
 
   public static void initDriver() {
-    synchronized (TestUtil.class) {
+    try (ResourceLock ignore = lock.obtain()) {
       if (initialized) {
         return;
       }
@@ -304,6 +313,10 @@ public class TestUtil {
    *         functions now as of 4/14
    */
   public static Connection openPrivilegedDB() throws SQLException {
+    return openPrivilegedDB(getDatabase());
+  }
+
+  public static Connection openPrivilegedDB(String databaseName) throws SQLException {
     initDriver();
     Properties properties = new Properties();
 
@@ -311,7 +324,7 @@ public class TestUtil {
     PGProperty.USER.set(properties, getPrivilegedUser());
     PGProperty.PASSWORD.set(properties, getPrivilegedPassword());
     PGProperty.OPTIONS.set(properties, "-c synchronous_commit=on");
-    return DriverManager.getConnection(getURL(), properties);
+    return DriverManager.getConnection(getURL(databaseName), properties);
 
   }
 
@@ -1144,15 +1157,6 @@ public class TestUtil {
     }
   }
 
-  /**
-   * Executes given SQL via {@link Statement#execute(String)} on a given connection.
-   * @deprecated prefer {@link #execute(Connection, String)} since it yields easier for read code
-   */
-  @Deprecated
-  public static void execute(String sql, Connection connection) throws SQLException {
-    execute(connection, sql);
-  }
-
   /**
    * Executes given SQL via {@link Statement#execute(String)} on a given connection.
    */
