@@ -17,6 +17,7 @@ import org.postgresql.core.ServerVersion;
 import org.postgresql.jdbc.PgStatement;
 import org.postgresql.test.TestUtil;
 import org.postgresql.test.util.StrangeProxyServer;
+import org.postgresql.util.LazyCleaner;
 import org.postgresql.util.PSQLState;
 
 import org.junit.After;
@@ -32,6 +33,7 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.sql.SQLWarning;
 import java.sql.Statement;
+import java.time.Duration;
 import java.util.ArrayList;
 import java.util.HashMap;
 import java.util.List;
@@ -43,6 +45,7 @@ import java.util.concurrent.ExecutionException;
 import java.util.concurrent.ExecutorService;
 import java.util.concurrent.Executors;
 import java.util.concurrent.Future;
+import java.util.concurrent.ThreadLocalRandom;
 import java.util.concurrent.TimeUnit;
 import java.util.concurrent.TimeoutException;
 import java.util.concurrent.atomic.AtomicInteger;
@@ -74,8 +77,8 @@ public class StatementTest {
     TestUtil.dropTable(con, "escapetest");
     TestUtil.dropTable(con, "comparisontest");
     TestUtil.dropTable(con, "test_lock");
-    TestUtil.execute("DROP FUNCTION IF EXISTS notify_loop()",con);
-    TestUtil.execute("DROP FUNCTION IF EXISTS notify_then_sleep()",con);
+    TestUtil.execute(con, "DROP FUNCTION IF EXISTS notify_loop()");
+    TestUtil.execute(con, "DROP FUNCTION IF EXISTS notify_then_sleep()");
     con.close();
   }
 
@@ -1059,6 +1062,11 @@ public class StatementTest {
 
     final AtomicInteger leaks = new AtomicInteger();
     final AtomicReference<Throwable> cleanupFailure = new AtomicReference<Throwable>();
+    // Create several cleaners, so they can clean leaks concurrently
+    List<LazyCleaner> cleaners = new ArrayList<>();
+    for (int i = 0; i < 16; i++) {
+      cleaners.add(new LazyCleaner(Duration.ofSeconds(2), "pgjdbc-test-cleaner-" + i));
+    }
 
     for (int q = 0; System.nanoTime() < deadline || leaks.get() < 10000; q++) {
       for (int i = 0; i < 100; i++) {
@@ -1066,20 +1074,16 @@ public class StatementTest {
         ps.close();
       }
       final int nextId = q;
-      new Object() {
-        PreparedStatement ps = con.prepareStatement("select /*leak*/ " + nextId);
-
-        @Override
-        protected void finalize() throws Throwable {
-          super.finalize();
-          try {
-            ps.close();
-          } catch (Throwable t) {
-            cleanupFailure.compareAndSet(null, t);
-          }
-          leaks.incrementAndGet();
+      int cleanerId = ThreadLocalRandom.current().nextInt(cleaners.size());
+      PreparedStatement ps = con.prepareStatement("select /*leak*/ " + nextId);
+      cleaners.get(cleanerId).register(new Object(), leak -> {
+        try {
+          ps.close();
+        } catch (Throwable t) {
+          cleanupFailure.compareAndSet(null, t);
         }
-      };
+        leaks.incrementAndGet();
+      });
     }
     if (cleanupFailure.get() != null) {
       throw new IllegalStateException("Detected failure in cleanup thread", cleanupFailure.get());
